package mods.immibis.connectedglass;

import java.io.IOException;

import net.minecraft.client.renderer.texture.IconRegister;
import net.minecraft.client.renderer.texture.TextureAtlasSprite;
import net.minecraft.client.renderer.texture.TextureMap;
import net.minecraft.client.resources.ResourceManager;
import net.minecraft.util.Icon;
import net.minecraft.util.ResourceLocation;
import net.minecraft.world.IBlockAccess;

public class Hooks {
	
	public static class TextureNoEdge extends TextureAtlasSprite {

		private ResourceLocation baseResLoc;
		private int mask;
		
		public TextureNoEdge(String name, ResourceLocation base, int mask) {
			super(name);
			
			this.baseResLoc = base;
			this.mask = mask;
		}
		
		@Override
		public boolean load(ResourceManager manager, ResourceLocation location) throws IOException {
			if(!super.load(manager, baseResLoc))
				return false;
			
			if(framesTextureData.size() == 0)
				// shouldn't happen
				return true;
			
			int[] data = (int[])framesTextureData.get(0);
			
			if((mask & MASK_TOP) != 0)
				for(int k = 1; k < width-1; k++)
					data[k] = 0;//data[k+width];
			
			if((mask & MASK_BOTTOM) != 0)
				for(int k = 2; k < width; k++)
					data[data.length-k] = 0;//data[data.length-k-width];
			
			if((mask & MASK_LEFT) != 0)
				for(int k = 1; k < height-1; k++)
					data[k*width] = 0;//data[k*width+1];
			
			if((mask & MASK_RIGHT) != 0)
				for(int k = 2; k < height; k++)
					data[k*width-1] = 0;//data[k*width-2];
			
			if((mask & MASK_TOPRIGHT) == MASK_TOPRIGHT)
				data[width-1] = 0;
			
			if((mask & MASK_TOPLEFT) == MASK_TOPLEFT)
				data[0] = 0;
			
			if((mask & MASK_BOTRIGHT) == MASK_BOTRIGHT)
				data[width*height-1] = 0;
			
			if((mask & MASK_BOTLEFT) == MASK_BOTLEFT)
				data[width*(height-1)] = 0;
			
			return true;
		}

	}

	private static final int MASK_TOP = 1;
	private static final int MASK_LEFT = 2;
	private static final int MASK_RIGHT = 4;
	private static final int MASK_BOTTOM = 8;
	private static final int MASK_TOPLEFT = MASK_TOP | MASK_LEFT;
	private static final int MASK_TOPRIGHT = MASK_TOP | MASK_RIGHT;
	private static final int MASK_BOTLEFT = MASK_BOTTOM | MASK_LEFT;
	private static final int MASK_BOTRIGHT = MASK_BOTTOM | MASK_RIGHT;
	
	private static TextureNoEdge[] i = new TextureNoEdge[16];
	
	private static Icon getTexture(boolean top, boolean bottom, boolean left, boolean right) {
		return i[(top ? MASK_TOP : 0) + (bottom ? MASK_BOTTOM : 0) + (left ? MASK_LEFT : 0) + (right ? MASK_RIGHT : 0)];
	}
	
	private static Icon getSideTexture(IBlockAccess w, int x, int y, int z, int hdx, int hdz) {
		int id = w.getBlockId(x, y, z);
		return getTexture(w.getBlockId(x,y+1,z) == id, w.getBlockId(x,y-1,z) == id, w.getBlockId(x+hdx,y,z+hdz) == id, w.getBlockId(x-hdx, y, z-hdz) == id);
	}
	
	public static Icon getBlockTexture(IBlockAccess w, int x, int y, int z, int side) {
		switch(side) {
		case 0:
		case 1:
			int id = w.getBlockId(x, y, z);
			return getTexture(w.getBlockId(x,y,z-1)==id, w.getBlockId(x,y,z+1)==id, w.getBlockId(x-1,y,z)==id, w.getBlockId(x+1,y,z)==id);
		case 2: return getSideTexture(w, x, y, z, 1, 0);
		case 3: return getSideTexture(w, x, y, z, -1, 0);
		case 4: return getSideTexture(w, x, y, z, 0, -1);
		case 5: return getSideTexture(w, x, y, z, 0, 1);
		default: return i[0];
		}
	}
	
	public static void registerIcons(IconRegister r) {
		ResourceLocation base = new ResourceLocation("textures/blocks/glass.png");
		for(int k = 0; k < 16; k++) {
			String name = "connectedglass:glass"+k;
			((TextureMap)r).setTextureEntry(name, i[k] = new TextureNoEdge(name, base, k));
		}
	}
}
